@@ -18,7 +18,7 @@ async fn main() -> Result<(), Error> {
     .await?;
     let dbname = db_name_new();
     let db: Database = client.database(&dbname);
-    let bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
+    let mut bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
     let id = bucket
         .upload_from_stream("test.txt", "test data".as_bytes(), None)
         .await?;
