@@ -39,7 +39,11 @@ impl GridFSBucket {
     # }
     ```
      */
-    pub async fn find(&self, filter: Document, options: GridFSFindOptions) -> Result<Cursor<Document>> {
+    pub async fn find(
+        &self,
+        filter: Document,
+        options: GridFSFindOptions,
+    ) -> Result<Cursor<Document>> {
         let dboptions = self.options.clone().unwrap_or_default();
         let bucket_name = dboptions.bucket_name;
         let file_collection = bucket_name + ".files";
