@@ -2,29 +2,28 @@ use crate::bucket::GridFSBucket;
 use crate::options::GridFSUploadOptions;
 use bson::{doc, oid::ObjectId, Document};
 use chrono::Utc;
+use futures::io::{AsyncRead, AsyncReadExt};
 use md5::{Digest, Md5};
 use mongodb::{
     error::Error,
     options::{FindOneOptions, InsertOneOptions, UpdateOptions},
     Collection,
 };
-use futures::StreamExt;
-use std::io::Read;
 
 impl GridFSBucket {
     async fn create_files_index(&self, collection_name: &str) -> Result<Document, Error> {
         self.db
             .run_command(
                 doc! {
-                    "createIndexes": collection_name,
-                    "indexes": [
-                        {
-                            "key": {
-                "filename":1,
-                "uploadDate":1.0
-                            },
-                            "name": collection_name.to_owned()+"_index",
-                    }]},
+                "createIndexes": collection_name,
+                "indexes": [
+                    {
+                        "key": {
+                            "filename":1,
+                            "uploadDate":1.0
+                        },
+                        "name": collection_name.to_owned()+"_index",
+                }]},
                 None,
             )
             .await
@@ -116,7 +115,7 @@ impl GridFSBucket {
                         }
                     }
                     if !have_index {
-                        self.create_files_index(&file_collection).await?;
+                        self.create_files_index(file_collection).await?;
                     }
                 }
                 {
@@ -167,7 +166,7 @@ impl GridFSBucket {
                         }
                     }
                     if !have_index {
-                        self.create_chunks_index(&chunk_collection).await?;
+                        self.create_chunks_index(chunk_collection).await?;
                     }
                 }
             }
@@ -207,7 +206,7 @@ impl GridFSBucket {
        #    let client = Client::with_uri_str(&std::env::var("MONGO_URI").unwrap_or("mongodb://localhost:27017/".to_string())).await?;
        #    let dbname = db_name_new();
        #    let db: Database = client.database(&dbname);
-       let bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
+       let mut bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
        let id = bucket
            .upload_from_stream("test.txt", "stream your data here".as_bytes(), None)
            .await?;
@@ -217,9 +216,9 @@ impl GridFSBucket {
        ```
     */
     pub async fn upload_from_stream<'a>(
-        mut self,
+        &mut self,
         filename: &str,
-        mut source: impl Read,
+        mut source: impl AsyncRead + Unpin,
         options: Option<GridFSUploadOptions>,
     ) -> Result<ObjectId, Error> {
         let dboptions = self.options.clone().unwrap_or_default();
@@ -264,7 +263,7 @@ impl GridFSBucket {
         let mut n: u32 = 0;
         loop {
             let buffer = vecbuf.as_mut_slice();
-            let read_size = source.read(buffer)?;
+            let read_size = source.read(buffer).await?;
             if read_size == 0 {
                 break;
             }
@@ -302,7 +301,7 @@ impl GridFSBucket {
             )
             .await?;
 
-        Ok(files_id.clone())
+        Ok(files_id)
     }
 }
 
@@ -311,9 +310,9 @@ mod tests {
     use super::GridFSBucket;
     use crate::options::GridFSBucketOptions;
     use bson::{doc, Document};
+    use futures::StreamExt;
     use mongodb::Client;
     use mongodb::{error::Error, Database};
-    use futures::StreamExt;
     use uuid::Uuid;
     fn db_name_new() -> String {
         "test_".to_owned()
@@ -330,7 +329,7 @@ mod tests {
         .await?;
         let dbname = db_name_new();
         let db: Database = client.database(&dbname);
-        let bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
+        let mut bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
         let id = bucket
             .upload_from_stream("test.txt", "test data".as_bytes(), None)
             .await?;
@@ -338,7 +337,7 @@ mod tests {
         assert_eq!(id.to_hex(), id.to_hex());
         let file = db
             .collection::<Document>("fs.files")
-            .find_one(doc! { "_id": id.clone() }, None)
+            .find_one(doc! { "_id": id }, None)
             .await?
             .unwrap();
         assert_eq!(file.get_str("filename").unwrap(), "test.txt");
@@ -363,7 +362,7 @@ mod tests {
                 .unwrap()
                 .get_binary_generic("data")
                 .unwrap(),
-            &vec![116 as u8, 101, 115, 116, 32, 100, 97, 116, 97]
+            &vec![116_u8, 101, 115, 116, 32, 100, 97, 116, 97]
         );
 
         db.drop(None).await
@@ -378,7 +377,7 @@ mod tests {
         .await?;
         let dbname = db_name_new();
         let db: Database = client.database(&dbname);
-        let bucket = GridFSBucket::new(
+        let mut bucket = GridFSBucket::new(
             db.clone(),
             Some(GridFSBucketOptions::builder().chunk_size_bytes(8).build()),
         );
@@ -389,7 +388,7 @@ mod tests {
         assert_eq!(id.to_hex(), id.to_hex());
         let file = db
             .collection::<Document>("fs.files")
-            .find_one(doc! { "_id": id.clone() }, None)
+            .find_one(doc! { "_id": id }, None)
             .await?
             .unwrap();
         assert_eq!(file.get_str("filename").unwrap(), "test.txt");
@@ -414,7 +413,7 @@ mod tests {
                 .unwrap()
                 .get_binary_generic("data")
                 .unwrap(),
-            &vec![116 as u8, 101, 115, 116, 32, 100, 97, 116]
+            &vec![116_u8, 101, 115, 116, 32, 100, 97, 116]
         );
 
         assert_eq!(chunks[1].as_ref().unwrap().get_i32("n").unwrap(), 1);
@@ -424,7 +423,7 @@ mod tests {
                 .unwrap()
                 .get_binary_generic("data")
                 .unwrap(),
-            &vec![97 as u8, 32, 49, 50, 51, 52, 53, 54]
+            &vec![97_u8, 32, 49, 50, 51, 52, 53, 54]
         );
 
         assert_eq!(chunks[2].as_ref().unwrap().get_i32("n").unwrap(), 2);
@@ -434,7 +433,7 @@ mod tests {
                 .unwrap()
                 .get_binary_generic("data")
                 .unwrap(),
-            &vec![55 as u8, 56, 57, 48]
+            &vec![55_u8, 56, 57, 48]
         );
 
         db.drop(None).await
@@ -449,7 +448,7 @@ mod tests {
         .await?;
         let dbname = db_name_new();
         let db: Database = client.database(&dbname);
-        let bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
+        let mut bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
 
         let indexes = db
             .run_command(doc! {"listIndexes":"fs.files"}, None)
@@ -510,7 +509,7 @@ mod tests {
         .await?;
         let dbname = db_name_new();
         let db: Database = client.database(&dbname);
-        let bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
+        let mut bucket = GridFSBucket::new(db.clone(), Some(GridFSBucketOptions::default()));
 
         let indexes = db
             .run_command(doc! {"listIndexes":"fs.chunks"}, None)
